/****************************************************************************************
Mat 259 : Project 2 3D Visualization
"Lifespan of Books at the Seattle Public Library"
                                                     
Author: Brianna Griffin                        
Supervisor: George Legrady                           

Purpose: Show how to present library data in a basic 3D environment. Show relationships 
         between the first checkout date and the last checkin date at the SPL. 

Usage:    1. A mouse left-drag will rotate the camera around the subject. 
          2. A double-click restores the camera to its original position. 
          3. 'p' - removes/adds points
          4. '1' - removes/adds data with 1 ≤ checkouts ≤ 10
          5. '2' - removes/adds data with 11 ≤ checkouts ≤ 20
          6. '3' - removes/adds data with 21 ≤ checkouts ≤ 30
          7. '4' - removes/adds data with 31 ≤ checkouts ≤ 40
          8. '5' - removes/adds data with checkouts ≥ 41
          9. 'x' - removes text on screen
          10. '<' & '>': rotate the X axis
          11. 'w' & 's': rotate the Y axis
          12. 'o' & 'i': rotate the Z axis
(color of the vector denotes the number of checkouts of the book since 2005 at the SPL)
****************************************************************************************/

import peasy.*; 
import controlP5.*;

ControlP5 cp5; 
PeasyCam cam;

Table table;
int numRows; 
ArrayList<Item> items = new ArrayList<Item>();
int numMiddlePoints = 10;


void setup(){
  size(1500, 800, P3D); 
  cam = new PeasyCam(this, 200); 
  cam.setMinimumDistance(60); 
  cam.setMaximumDistance(150);  
  cam.lookAt(6, 15, 15); // sets center POV
  
  float fov = PI/10; // field of view try between PI/2 to PI/10
  
  float  cameraZ = (height/2.0) / tan(PI*60.0/360.0);

  //perspective(field of view in radians; aspect/ratio of width to height; zNear - Z position of nearest clipping, zFar - Z position of farthest clipping)
  //default values are : perspective(PI/3.0, width/height, cameraZ/10.0, cameraZ*10.0) where cameraZ is ((height/2.0) / tan(PI*60.0/360.0));
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*10.0); // if using 0.001 instead of cameraZ/10.0 so that all things close to the screen can be seen - get noise
  
  // import the csv 
  table = loadTable("data_new.csv", "header");
  numRows = table.getRowCount();
  
  // make the array with the data from the csv 
  for(int i=0; i<numRows; i++){
    String title = table.getString(i,0);
    float x_0 = table.getFloat(i,1);
    float y_0 = table.getFloat(i,2);
    float z_0 = table.getFloat(i,3);
    float x_1 = table.getFloat(i,4); 
    float y_1 = table.getFloat(i,4); 
    float z_1 = table.getFloat(i,6); 
    String item_cat = table.getString(i,7); 
    float number_checkouts = table.getFloat(i,8); 
    items.add(new Item(title,x_0, y_0, z_0, x_1, y_1, z_1, item_cat, number_checkouts));
  }
}

void draw(){
  background(200);
  
for(int i = 0; i<numRows; i++){ 
  
  if(A){
  if(items.get(i).number_checkouts >= 1 && items.get(i).number_checkouts <= 10){
  line(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0, 
  items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  color middleColor;
  middleColor = lerpColor(begColor(items.get(i).number_checkouts),endColor(items.get(i).number_checkouts), i/float(numRows));
  stroke(middleColor);}
  }
  
  if(B){
  if(items.get(i).number_checkouts >= 11 && items.get(i).number_checkouts <= 20){
  line(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0, 
  items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  color middleColor;
  middleColor = lerpColor(begColor(items.get(i).number_checkouts),endColor(items.get(i).number_checkouts), i/float(numRows));
  stroke(middleColor);}
  }
  
  if(C){
  if(items.get(i).number_checkouts >= 21 && items.get(i).number_checkouts <= 30){
  line(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0, 
  items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  color middleColor;
  middleColor = lerpColor(begColor(items.get(i).number_checkouts),endColor(items.get(i).number_checkouts), i/float(numRows));
  stroke(middleColor);}
  }
  
  if(D){
  if(items.get(i).number_checkouts >= 31 && items.get(i).number_checkouts <= 40){
  line(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0, 
  items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  color middleColor;
  middleColor = lerpColor(begColor(items.get(i).number_checkouts),endColor(items.get(i).number_checkouts), i/float(numRows));
  stroke(middleColor);}    
  }
  
  if(E){
  if(items.get(i).number_checkouts >= 41){
  line(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0, 
  items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  color middleColor;
  middleColor = lerpColor(begColor(items.get(i).number_checkouts),endColor(items.get(i).number_checkouts), i/float(numRows));
  stroke(middleColor);}    
  }
  
  }
  
  
 pushStyle();

for(int i = 0; i<numRows; i++){ 
  
  if (points){
    
  if(pointA){
  if(items.get(i).number_checkouts >= 1 && items.get(i).number_checkouts <= 10){
  color beg = begColor(items.get(i).number_checkouts);
  color end = endColor(items.get(i).number_checkouts);
  point(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0);
  stroke(beg);
  strokeWeight(6);
  point(items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  stroke(end);
  strokeWeight(6);
    }}
  
  if(pointB){
  if(items.get(i).number_checkouts >= 11 && items.get(i).number_checkouts <= 20){
  color beg = begColor(items.get(i).number_checkouts);
  color end = endColor(items.get(i).number_checkouts);
  point(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0);
  stroke(beg);
  strokeWeight(6);
  point(items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  stroke(end);
  strokeWeight(6);}}
  
  if(pointC){
  if(items.get(i).number_checkouts >= 21 && items.get(i).number_checkouts <= 30){
  color beg = begColor(items.get(i).number_checkouts);
  color end = endColor(items.get(i).number_checkouts);
  point(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0);
  stroke(beg);
  strokeWeight(6);
  point(items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  stroke(end);
  strokeWeight(6);}}  
  
  if(pointD){
  if(items.get(i).number_checkouts >= 31 && items.get(i).number_checkouts <= 40){  
  color beg = begColor(items.get(i).number_checkouts);
  color end = endColor(items.get(i).number_checkouts);
  point(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0);
  stroke(beg);
  strokeWeight(6);
  point(items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  stroke(end);
  strokeWeight(6);}}
  
  if(pointE){
  if(items.get(i).number_checkouts >= 41){
  color beg = begColor(items.get(i).number_checkouts);
  color end = endColor(items.get(i).number_checkouts);
  point(items.get(i).x_0, items.get(i).y_0, items.get(i).z_0);
  stroke(beg);
  strokeWeight(6);
  point(items.get(i).x_1, items.get(i).y_1, items.get(i).z_1);
  stroke(end);
  strokeWeight(6);}}  
  }
}
    
  popStyle(); 
  hi(); 

  // x axis
  if(textselect){
  stroke(110);
  line(0, 0, 0, 12, 0, 0);
  // y axis 
  line(0, 0, 0, 0, 31, 0); 
  // z axis
  line(0, 0, 0, 0, 0, 23);
  
  textAxis();
  }

}
